@@ -6,16 +6,13 @@ non-local variables defined in outer scopes.
 
 from typing import List
 
-from mypy.nodes import Var
-
 from mypyc.common import SELF_NAME, ENV_ATTR_NAME
-from mypyc.ir.ops import BasicBlock, Return, Call, SetAttr, Value, Environment
+from mypyc.ir.ops import BasicBlock, Return, Call, SetAttr, Value, Register
 from mypyc.ir.rtypes import RInstance, object_rprimitive
 from mypyc.ir.func_ir import FuncIR, FuncSignature, RuntimeArg, FuncDecl
 from mypyc.ir.class_ir import ClassIR
 from mypyc.irbuild.builder import IRBuilder
 from mypyc.irbuild.context import FuncInfo, ImplicitClass
-from mypyc.irbuild.util import add_self_to_env
 from mypyc.primitives.misc_ops import method_new_op
 
 
@@ -79,26 +76,25 @@ def setup_callable_class(builder: IRBuilder) -> None:
 
     # Add a 'self' variable to the environment of the callable class,
     # and store that variable in a register to be accessed later.
-    self_target = add_self_to_env(builder.environment, callable_class_ir)
+    self_target = builder.add_self_to_env(callable_class_ir)
     builder.fn_info.callable_class.self_reg = builder.read(self_target, builder.fn_info.fitem.line)
 
 
 def add_call_to_callable_class(builder: IRBuilder,
+                               args: List[Register],
                                blocks: List[BasicBlock],
                                sig: FuncSignature,
-                               env: Environment,
                                fn_info: FuncInfo) -> FuncIR:
     """Generate a '__call__' method for a callable class representing a nested function.
 
-    This takes the blocks, signature, and environment associated with
-    a function definition and uses those to build the '__call__'
-    method of a given callable class, used to represent that
-    function.
+    This takes the blocks and signature associated with a function
+    definition and uses those to build the '__call__' method of a
+    given callable class, used to represent that function.
     """
     # Since we create a method, we also add a 'self' parameter.
     sig = FuncSignature((RuntimeArg(SELF_NAME, object_rprimitive),) + sig.args, sig.ret_type)
     call_fn_decl = FuncDecl('__call__', fn_info.callable_class.ir.name, builder.module_name, sig)
-    call_fn_ir = FuncIR(call_fn_decl, blocks, env,
+    call_fn_ir = FuncIR(call_fn_decl, args, blocks,
                         fn_info.fitem.line, traceback_name=fn_info.fitem.name)
     fn_info.callable_class.ir.methods['__call__'] = call_fn_ir
     return call_fn_ir
@@ -107,39 +103,31 @@ def add_call_to_callable_class(builder: IRBuilder,
 def add_get_to_callable_class(builder: IRBuilder, fn_info: FuncInfo) -> None:
     """Generate the '__get__' method for a callable class."""
     line = fn_info.fitem.line
-    builder.enter(fn_info)
 
-    vself = builder.read(
-        builder.environment.add_local_reg(Var(SELF_NAME), object_rprimitive, True)
+    builder.enter_method(
+        fn_info.callable_class.ir, '__get__', object_rprimitive, fn_info,
+        self_type=object_rprimitive
     )
-    instance = builder.environment.add_local_reg(Var('instance'), object_rprimitive, True)
-    builder.environment.add_local_reg(Var('owner'), object_rprimitive, True)
+    instance = builder.add_argument('instance', object_rprimitive)
+    builder.add_argument('owner', object_rprimitive)
 
     # If accessed through the class, just return the callable
     # object. If accessed through an object, create a new bound
     # instance method object.
     instance_block, class_block = BasicBlock(), BasicBlock()
-    comparison = builder.binary_op(
+    comparison = builder.translate_is_op(
         builder.read(instance), builder.none_object(), 'is', line
     )
     builder.add_bool_branch(comparison, class_block, instance_block)
 
     builder.activate_block(class_block)
-    builder.add(Return(vself))
+    builder.add(Return(builder.self()))
 
     builder.activate_block(instance_block)
-    builder.add(Return(builder.call_c(method_new_op, [vself, builder.read(instance)], line)))
+    builder.add(Return(builder.call_c(method_new_op,
+                                      [builder.self(), builder.read(instance)], line)))
 
-    blocks, env, _, fn_info = builder.leave()
-
-    sig = FuncSignature((RuntimeArg(SELF_NAME, object_rprimitive),
-                         RuntimeArg('instance', object_rprimitive),
-                         RuntimeArg('owner', object_rprimitive)),
-                        object_rprimitive)
-    get_fn_decl = FuncDecl('__get__', fn_info.callable_class.ir.name, builder.module_name, sig)
-    get_fn_ir = FuncIR(get_fn_decl, blocks, env)
-    fn_info.callable_class.ir.methods['__get__'] = get_fn_ir
-    builder.functions.append(get_fn_ir)
+    builder.leave_method()
 
 
 def instantiate_callable_class(builder: IRBuilder, fn_info: FuncInfo) -> Value:
