@@ -4,17 +4,17 @@ from typing import List, Optional
 
 from mypy.nodes import (
     ClassDef, FuncDef, OverloadedFuncDef, PassStmt, AssignmentStmt, NameExpr, StrExpr,
-    ExpressionStmt, TempNode, Decorator, Lvalue, RefExpr, Var, is_class_var
+    ExpressionStmt, TempNode, Decorator, Lvalue, RefExpr, is_class_var
 )
 from mypyc.ir.ops import (
-    Value, Call, LoadErrorValue, LoadStatic, InitStatic, TupleSet, SetAttr, Return,
+    Value, Register, Call, LoadErrorValue, LoadStatic, InitStatic, TupleSet, SetAttr, Return,
     BasicBlock, Branch, MethodCall, NAMESPACE_TYPE, LoadAddress
 )
 from mypyc.ir.rtypes import (
-    RInstance, object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
+    object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
     is_object_rprimitive, is_none_rprimitive
 )
-from mypyc.ir.func_ir import FuncIR, FuncDecl, FuncSignature, RuntimeArg
+from mypyc.ir.func_ir import FuncDecl, FuncSignature
 from mypyc.ir.class_ir import ClassIR, NonExtClassInfo
 from mypyc.primitives.generic_ops import py_setattr_op, py_hasattr_op
 from mypyc.primitives.misc_ops import (
@@ -22,9 +22,8 @@ from mypyc.primitives.misc_ops import (
     not_implemented_op
 )
 from mypyc.primitives.dict_ops import dict_set_item_op, dict_new_op
-from mypyc.common import SELF_NAME
 from mypyc.irbuild.util import (
-    is_dataclass_decorator, get_func_def, is_dataclass, is_constant, add_self_to_env
+    is_dataclass_decorator, get_func_def, is_dataclass, is_constant
 )
 from mypyc.irbuild.builder import IRBuilder
 from mypyc.irbuild.function import transform_method
@@ -171,8 +170,8 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
     template = builder.add(LoadStatic(object_rprimitive, cdef.name + "_template",
                                    builder.module_name, NAMESPACE_TYPE))
     # Create the class
-    tp = builder.primitive_op(pytype_from_template_op,
-                           [template, tp_bases, modname], cdef.line)
+    tp = builder.call_c(pytype_from_template_op,
+                    [template, tp_bases, modname], cdef.line)
     # Immediately fix up the trait vtables, before doing anything with the class.
     ir = builder.mapper.type_to_ir[cdef.info]
     if not ir.is_trait and not ir.builtin_base:
@@ -229,7 +228,7 @@ def find_non_ext_metaclass(builder: IRBuilder, cdef: ClassDef, bases: Value) ->
         declared_metaclass = builder.add(LoadAddress(type_object_op.type,
                                                      type_object_op.src, cdef.line))
 
-    return builder.primitive_op(py_calc_meta_op, [declared_metaclass, bases], cdef.line)
+    return builder.call_c(py_calc_meta_op, [declared_metaclass, bases], cdef.line)
 
 
 def setup_non_ext_dict(builder: IRBuilder,
@@ -245,7 +244,7 @@ def setup_non_ext_dict(builder: IRBuilder,
                                 [metaclass,
                                 builder.load_static_unicode('__prepare__')], cdef.line)
 
-    non_ext_dict = builder.alloc_temp(dict_rprimitive)
+    non_ext_dict = Register(dict_rprimitive)
 
     true_block, false_block, exit_block, = BasicBlock(), BasicBlock(), BasicBlock()
     builder.add_bool_branch(has_prepare, true_block, false_block)
@@ -327,12 +326,9 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef) -> None:
     if not default_assignments:
         return
 
-    builder.enter()
-    builder.ret_types[-1] = bool_rprimitive
-
-    rt_args = (RuntimeArg(SELF_NAME, RInstance(cls)),)
-    self_var = builder.read(add_self_to_env(builder.environment, cls), -1)
+    builder.enter_method(cls, '__mypyc_defaults_setup', bool_rprimitive)
 
+    self_var = builder.self()
     for stmt in default_assignments:
         lvalue = stmt.lvalues[0]
         assert isinstance(lvalue, NameExpr)
@@ -351,55 +347,31 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef) -> None:
 
     builder.add(Return(builder.true()))
 
-    blocks, env, ret_type, _ = builder.leave()
-    ir = FuncIR(
-        FuncDecl('__mypyc_defaults_setup',
-                 cls.name, builder.module_name,
-                 FuncSignature(rt_args, ret_type)),
-        blocks, env)
-    builder.functions.append(ir)
-    cls.methods[ir.name] = ir
+    builder.leave_method()
 
 
 def create_ne_from_eq(builder: IRBuilder, cdef: ClassDef) -> None:
     """Create a "__ne__" method from a "__eq__" method (if only latter exists)."""
     cls = builder.mapper.type_to_ir[cdef.info]
     if cls.has_method('__eq__') and not cls.has_method('__ne__'):
-        f = gen_glue_ne_method(builder, cls, cdef.line)
-        cls.method_decls['__ne__'] = f.decl
-        cls.methods['__ne__'] = f
-        builder.functions.append(f)
+        gen_glue_ne_method(builder, cls, cdef.line)
 
 
-def gen_glue_ne_method(builder: IRBuilder, cls: ClassIR, line: int) -> FuncIR:
+def gen_glue_ne_method(builder: IRBuilder, cls: ClassIR, line: int) -> None:
     """Generate a "__ne__" method from a "__eq__" method. """
-    builder.enter()
-
-    rt_args = (RuntimeArg("self", RInstance(cls)), RuntimeArg("rhs", object_rprimitive))
-
-    # The environment operates on Vars, so we make some up
-    fake_vars = [(Var(arg.name), arg.type) for arg in rt_args]
-    args = [
-        builder.read(
-            builder.environment.add_local_reg(
-                var, type, is_arg=True
-            ),
-            line
-        )
-        for var, type in fake_vars
-    ]  # type: List[Value]
-    builder.ret_types[-1] = object_rprimitive
+    builder.enter_method(cls, '__ne__', object_rprimitive)
+    rhs_arg = builder.add_argument('rhs', object_rprimitive)
 
     # If __eq__ returns NotImplemented, then __ne__ should also
     not_implemented_block, regular_block = BasicBlock(), BasicBlock()
-    eqval = builder.add(MethodCall(args[0], '__eq__', [args[1]], line))
+    eqval = builder.add(MethodCall(builder.self(), '__eq__', [rhs_arg], line))
     not_implemented = builder.add(LoadAddress(not_implemented_op.type,
                                               not_implemented_op.src, line))
     builder.add(Branch(
-        builder.binary_op(eqval, not_implemented, 'is', line),
+        builder.translate_is_op(eqval, not_implemented, 'is', line),
         not_implemented_block,
         regular_block,
-        Branch.BOOL_EXPR))
+        Branch.BOOL))
 
     builder.activate_block(regular_block)
     retval = builder.coerce(
@@ -410,11 +382,7 @@ def gen_glue_ne_method(builder: IRBuilder, cls: ClassIR, line: int) -> FuncIR:
     builder.activate_block(not_implemented_block)
     builder.add(Return(not_implemented))
 
-    blocks, env, ret_type, _ = builder.leave()
-    return FuncIR(
-        FuncDecl('__ne__', cls.name, builder.module_name,
-                 FuncSignature(rt_args, ret_type)),
-        blocks, env)
+    builder.leave_method()
 
 
 def load_non_ext_class(builder: IRBuilder,
